module DDLParser
  module SharedRules
    module Constants
      include Parslet

      rule(:digit)      { match['0-9'] }
      rule(:integer)    { (str('-').maybe >> digit.repeat(1)).as(:integer) }
      rule(:float)      { (str('-').maybe >> digit.repeat(1) >> str('.') >> digit.repeat(1)).as(:float) }

      rule(:space)      { match('\s').repeat(1) }
      rule(:space?)     { space.maybe }
      rule(:newline)    { str("\n") }
      rule(:spaces)     { (space | (comment? >> newline)).repeat }
      rule(:comment?)   { (str('--') >> (newline.absent? >> any).repeat).maybe }

      rule(:comma)      { str(',') >> space? }
      rule(:lparen)     { str('(') >> space? }
      rule(:rparen)     { str(')') >> space? }

      rule(:boolean)    { (str('true') | str('false')).as(:boolean) }
      rule(:datetime)   { (digit.repeat(4) >> str('-') >>
          digit.repeat(2) >> str('-') >>
          digit.repeat(2) >> str('T') >>
          digit.repeat(2) >> str(':') >>
          digit.repeat(2) >> str(':') >>
          digit.repeat(2) >> str('Z')).as(:datetime) }


      rule(:string)     {
        (str("'") >>
            (
            str('\\') >> any |
                str("'").absent? >> any
            ).repeat >>
            str("'")).as(:string)
      }

      rule(:const)      {
        integer | float | string | boolean | datetime
      }

      rule(:identifier)         { match('\\w').repeat(1) }
      rule(:current_timestamp)  { str('current timestamp').as(:timestamp) }

    end
  end
end



