module DDLParser
  module SharedRules
    module DataTypes
      include Parslet
      include DDLParser::SharedRules::Constants

      # Build-in data types
      rule(:type_smallint)      {str('smallint')}
      rule(:type_integer)       {(str('integer')| str('int'))}
      rule(:type_bigint)        {str('bigint').as(:bigint)}
      rule(:type_decimal)       {(((str('decimal') | str('dec') | str('numeric') | str('num')) >>
          (lparen >> integer.as(:total) >> (comma >> integer.as(:scale)).maybe >> rparen).as(:precision).maybe)).as(:decimal)}
      rule(:type_float)         {(str('float') >> (lparen >> integer >> rparen).as(:precision)).as(:float)}
      rule(:type_real)          {str('real')}
      rule(:type_double)        {str('double')}
      rule(:type_char)          {((str('character') | str('char')) >> (lparen >> integer >> rparen).as(:length)).as(:char)}
      rule(:type_varchar)       {(str('varchar') >> (lparen >> integer >> rparen).as(:length)).as(:varchar)}
      rule(:type_clob)          {(str('clob') >> (lparen >> integer >> rparen).as(:length)).as(:clob)}
      rule(:type_date)          {str('date')}
      rule(:type_timestamp)     {str('timestamp')}
      rule(:type_time)          {type_timestamp.absent? >> str('time')}


      rule(:data_type)          { (type_smallint | type_integer | type_bigint | type_decimal | type_float | type_real |
                                   type_double | type_char | type_varchar | type_clob | type_date | type_time | type_timestamp).as(:data_type)}


    end
  end
end



