module DDLParser
  module SQL
    module DB2
      class SelectParser < Parslet::Parser

        include DDLParser::SharedRules::Constants
        include DDLParser::SharedRules::LogicalOperators

        rule(:term)       { const | item }

        rule(:bool_and)   { str('and') }
        rule(:bool_or)    { str('or') }

        rule(:identifier) { match('\\w').repeat(1) }

        rule(:function)   {
          identifier.as(:function) >> space? >>
              lparen >> arglist.as(:arguments) >> rparen
        }

        rule(:item)       { function | identifier | string }

        rule(:arglist)    {
          item.as(:item) >> (comma >> item.as(:item)).repeat
        }
        rule(:namelist)   {
          identifier.as(:name) >> (comma >> identifier.as(:name)).repeat
        }

        rule(:between_cond){
          term.as(:lhs) >> space? >> str('between').as(:op) >> space? >> (const.as(:btw_from) >> space? >> bool_and >> space? >> const.as(:btw_to)).as(:rhs)
        }

        rule(:single_cond){
          (term.as(:lhs) >> space? >> binop.as(:op) >> space? >> term.as(:rhs)) | between_cond
        }

        rule(:condition)  {
          single_cond.as(:lhs) >> space? >>
              ((bool_and.as(:op) | bool_or.as(:op)) >>
                  space? >>
                  condition.as(:rhs)).maybe
        }

        rule(:select_s)   {
          str('select').as(:op) >> space? >> (arglist | str('*')).as(:select)
        }
        rule(:from_s)     {
          str('from') >> space? >> namelist.as(:from)
        }
        rule(:where_s)    {
          str('where') >> space? >> condition.as(:where)
        }
        rule(:group_s)    {
          str('group') >> space? >> str('by') >> space? >> item.as(:group_by)
        }
        rule(:select)     {
          select_s >> space? >> from_s >> space? >>
              (where_s >> space? >> group_s.maybe | group_s).maybe
        }

        rule(:expression) { select } #| insert | create }
        root :expression
      end
    end
  end
end
